from dataclasses import asdict
from typing import Any, Literal

from ..utils import GoogleKeysDTO, GoogleServiceDTO, get_google_service
from .schemas import (
    ColoredDataResponse,
    CreateSheetReply,
    SheetRangeResponse,
    Spreadsheet,
    UpdatedResponse,
)

SPREADSHEET_SERVICE = GoogleServiceDTO(
    version="v4",
    service_name="sheets",
    scope="https://www.googleapis.com/auth/spreadsheets",
)


class GoogleSpreadsheet:
    def __init__(self, google_keys: GoogleKeysDTO) -> None:
        self.service = get_google_service(
            google_keys=asdict(google_keys),
            scope=SPREADSHEET_SERVICE.scope,
            service_name=SPREADSHEET_SERVICE.service_name,
            version=SPREADSHEET_SERVICE.version,
        )

    def get_spreadsheet(self, spreadsheet_id: str) -> Spreadsheet:
        result = self.service.spreadsheets().get(spreadsheetId=spreadsheet_id).execute()
        return Spreadsheet(**result)

    def create_sheet(
        self,
        spreadsheet_id: str,
        title: str,
        index: int = 0,
        row_count: int = 200,
        col_count: int = 200,
        red: float = 1,
        green: float = 1,
        blue: float = 0,
    ) -> CreateSheetReply:
        request_body = {
            "requests": [
                {
                    "addSheet": {
                        "properties": {
                            "title": title,
                            "tabColor": {"red": red, "green": green, "blue": blue},
                            "index": index,
                            "gridProperties": {
                                "rowCount": row_count,
                                "columnCount": col_count,
                            },
                        }
                    }
                }
            ]
        }
        result = (
            self.service.spreadsheets()
            .batchUpdate(
                spreadsheetId=spreadsheet_id,
                body=request_body,
            )
            .execute()
        )
        return CreateSheetReply(**result)

    def get_sheet(self, spreadsheet_id: str, sheet: str) -> SheetRangeResponse:
        result = (
            self.service.spreadsheets()
            .values()
            .batchGet(
                spreadsheetId=spreadsheet_id,
                ranges=sheet,
                valueRenderOption="FORMATTED_VALUE",
                dateTimeRenderOption="FORMATTED_STRING",
            )
            .execute()
        )
        return SheetRangeResponse(**result)

    def set_data(
        self,
        spreadsheet_id: str,
        range_sheet: int,
        values: list[list[Any]],
        major_dimension: Literal["COLUMNS", "ROWS"] = "ROWS",
    ) -> UpdatedResponse:
        request_body = {
            "valueInputOption": "USER_ENTERED",
            "data": [
                {
                    "range": range_sheet,
                    "majorDimension": major_dimension,
                    "values": values,
                }
            ],
        }
        result = (
            self.service.spreadsheets()
            .values()
            .batchUpdate(spreadsheetId=spreadsheet_id, body=request_body)
            .execute()
        )
        return UpdatedResponse(**result)

    def get_sheet_with_colors(
        self, ss_id: str, range_sheet: str
    ) -> ColoredDataResponse:
        params = {
            "spreadsheetId": ss_id,
            "ranges": range_sheet,
            "fields": "sheets(data(rowData(values(effectiveFormat/"
            "backgroundColor,formattedValue)),startColumn,"
            "startRow))",
        }
        result = self.service.spreadsheets().get(**params).execute()
        return ColoredDataResponse(**result)

    def get_data(self, spreadsheet_id: str, sheet: str) -> list[list[str]]:
        result = self.get_sheet(spreadsheet_id=spreadsheet_id, sheet=sheet)
        return result.valueRanges[0].values

    def clear_sheet_data(self, spreadsheet_id: str, sheet: str) -> None:
        self.service.spreadsheets().values().clear(
            spreadsheetId=spreadsheet_id,
            range=sheet,
            body={},
        ).execute()



from pydantic import BaseModel


class SpreadsheetProperties(BaseModel):
    title: str
    locale: str
    autoRecalc: str
    timeZone: str


class SheetProperties(BaseModel):
    sheetId: int
    title: str
    index: int
    sheetType: str
    hidden: bool = False
    gridProperties: dict | None = None
    tabColor: dict | None = None
    tabColorStyle: dict | None = None


class Sheet(BaseModel):
    properties: SheetProperties


class Spreadsheet(BaseModel):
    spreadsheetId: str
    spreadsheetUrl: str
    properties: SpreadsheetProperties
    sheets: list[Sheet]


class Action(BaseModel):
    addSheet: Sheet


class CreateSheetReply(BaseModel):
    spreadsheetId: str
    replies: list[Action]


class UpdatedResponseRange(BaseModel):
    spreadsheetId: str
    updatedRange: str
    updatedRows: int
    updatedColumns: int
    updatedCells: int


class UpdatedResponse(BaseModel):
    spreadsheetId: str
    totalUpdatedRows: int
    totalUpdatedColumns: int
    totalUpdatedCells: int
    totalUpdatedSheets: int
    responses: list[UpdatedResponseRange]


class RangeResponse(BaseModel):
    range: str
    majorDimension: str
    values: list[list[str]]


class SheetRangeResponse(BaseModel):
    spreadsheetId: str
    valueRanges: list[RangeResponse]


class Colors(BaseModel):
    red: float
    green: float
    blue: float


class ColorParams(BaseModel):
    backgroundColor: Colors


class ColoredData(BaseModel):
    formattedValue: str
    effectiveFormat: ColorParams


class ColoredRow(BaseModel):
    values: list[ColoredData | dict]


class ColoredRowData(BaseModel):
    rowData: list[ColoredRow]


class ColoredSheetResponse(BaseModel):
    data: list[ColoredRowData]


class ColoredDataResponse(BaseModel):
    sheets: list[ColoredSheetResponse]



import time
from dataclasses import asdict
from typing import Literal

from googleapiclient.errors import HttpError

from ..utils import GoogleKeysDTO, GoogleServiceDTO, get_google_service
from .schemas import FileResponse

DRIVE_SERVICE = GoogleServiceDTO(
    version="v3", service_name="drive", scope="https://www.googleapis.com/auth/drive"
)


class GoogleDrive:
    FIELDS = "id,webViewLink,name,mimeType,kind,parents"

    def __init__(self, google_keys: GoogleKeysDTO):
        self.service = get_google_service(
            google_keys=asdict(google_keys),
            scope=DRIVE_SERVICE.scope,
            service_name=DRIVE_SERVICE.service_name,
            version=DRIVE_SERVICE.version,
        )

    def make_new_folder(
        self,
        new_folder_title: str,
        parent_folder_id: str,
    ) -> FileResponse | None:
        data = {
            "name": new_folder_title,
            "parents": [parent_folder_id],
            "mimeType": "application/vnd.google-apps.folder",
        }
        try:
            result = (
                self.service.files().create(body=data, fields=self.FIELDS).execute()
            )
            return FileResponse(**result)
        except HttpError:
            return None

    def get_folder_by_id(self, folder_id: str) -> FileResponse | None:
        try:
            result = (
                self.service.files().get(fileId=folder_id, fields=self.FIELDS).execute()
            )
            return FileResponse(**result)
        except HttpError:
            return None

    def set_permissions_for_users_by_list(
        self,
        folder_id: str,
        user_email_list: list[str],
        permission: Literal["reader", "writer"] = "writer",
    ) -> None:
        """Выдает права на объект пользователям из списка"""
        batch = self.service.new_batch_http_request()
        for email in user_email_list:
            batch.add(
                self.service.permissions().create(
                    fileId=folder_id,
                    body={
                        "type": "user",
                        "role": permission,
                        "emailAddress": email,
                    },
                )
            )
        batch.execute()

    def set_permissions_for_anyone(
        self, folder_id: str, permission: Literal["reader", "writer"] = "reader"
    ) -> None:
        """Дать доступ к файлу любому в интернете"""
        batch = self.service.new_batch_http_request()
        batch.add(
            self.service.permissions().create(
                fileId=folder_id,
                body={
                    "type": "anyone",
                    "role": permission,
                },
            )
        )
        batch.execute()

    def get_files(self, folder_id: str) -> list[FileResponse]:
        files: list[FileResponse] = []
        page_token = None
        while True:
            response = (
                self.service.files()
                .list(
                    q=f"'{folder_id}' in parents",
                    spaces="drive",
                    fields="nextPageToken, "
                    "files(id,webViewLink,name,mimeType,kind,parents)",
                    pageSize=1000,
                    pageToken=page_token,
                )
                .execute()
            )
            files.extend([FileResponse(**f) for f in response.get("files", [])])
            page_token = response.get("nextPageToken", None)
            if page_token is None:
                break
            time.sleep(0.1)
        return files

    def get_all_folder_files_recursively(self, folder_id: str) -> list[FileResponse]:
        files = self.get_files(folder_id=folder_id)
        for file in files:
            if ".folder" in file.mimeType:
                files.extend(self.get_all_folder_files_recursively(folder_id=file.id))
        return files



from dataclasses import dataclass

import httplib2
from googleapiclient.discovery import Resource, build
from oauth2client.service_account import ServiceAccountCredentials


@dataclass
class GoogleServiceDTO:
    version: str
    service_name: str
    scope: str


@dataclass
class GoogleKeysDTO:
    type: str
    project_id: str
    private_key_id: str
    private_key: str
    client_email: str
    client_id: str
    auth_uri: str
    token_uri: str
    auth_provider_x509_cert_url: str
    client_x509_cert_url: str


def get_google_service(
    google_keys: dict[str, str], scope: list[str] | str, service_name: str, version: str
) -> Resource:
    credentials = ServiceAccountCredentials.from_json_keyfile_dict(
        keyfile_dict=google_keys,
        scopes=scope,
    )
    http_auth = credentials.authorize(httplib2.Http())
    return build(serviceName=service_name, version=version, http=http_auth)
